!slow-roll functions for the R-R^p inflation potential
!
!V(phi) = M^4 * exp(-2y) * [ exp(y) - 1 ]^(2p/(2p-1))
!
!y = phi/Mp * sqrt(2/3)


module rpi2sr
  use infprec, only : kp,tolkp,transfert
  use inftools, only : zbrent
  use rpicommon, only : rpiBig
  use rpicommon, only : rpi_norm_potential, rpi_norm_deriv_potential
  use rpicommon, only : rpi_norm_deriv_second_potential
  use rpicommon, only : rpi_epsilon_one, rpi_epsilon_two, rpi_epsilon_three
  use rpicommon, only : rpih_efold_primitive, rpih_x_trajectory, rpi_x_potmax
  use rpicommon, only : rpi_efold_primitive, find_rpi_x_trajectory
  use rpicommon, only : rpi_ln_omega4
  implicit none

  private

  public rpi2_norm_potential, rpi2_epsilon_one, rpi2_epsilon_two, rpi2_epsilon_three
  public rpi2_efold_primitive, rpi2_x_trajectory, rpi2_numacc_efoldmax, rpi2_numacc_xendmin
  public rpi2_norm_deriv_potential, rpi2_norm_deriv_second_potential
  public rpi2_ln_omega4

contains
 
  !returns V/M^4
  function rpi2_norm_potential(y,p)
    implicit none
    real(kp) :: rpi2_norm_potential
    real(kp), intent(in) :: y,p

    rpi2_norm_potential = rpi_norm_potential(y,p)

  end function rpi2_norm_potential



  !returns the first derivative of the potential with respect to y, divided by M^4
  function rpi2_norm_deriv_potential(y,p)
    implicit none
    real(kp) :: rpi2_norm_deriv_potential
    real(kp), intent(in) :: y,p

    rpi2_norm_deriv_potential = rpi_norm_deriv_potential(y,p)

  end function rpi2_norm_deriv_potential



  !returns the second derivative of the potential with respect to y, divided by M^4
  function rpi2_norm_deriv_second_potential(y,p)
    implicit none
    real(kp) :: rpi2_norm_deriv_second_potential
    real(kp), intent(in) :: y,p

    rpi2_norm_deriv_second_potential = rpi_norm_deriv_second_potential(y,p)

  end function rpi2_norm_deriv_second_potential


  function rpi2_ln_omega4(y,p)
    implicit none
    real(kp) :: rpi2_ln_omega4
    real(kp), intent(in) :: y,p

    rpi2_ln_omega4 = rpi_ln_omega4(y,p)

  end function rpi2_ln_omega4
    

  !epsilon_one(y)
  function rpi2_epsilon_one(y,p)    
    implicit none
    real(kp) :: rpi2_epsilon_one
    real(kp), intent(in) :: y,p


    rpi2_epsilon_one = rpi_epsilon_one(y,p)


  end function rpi2_epsilon_one


  !epsilon_two(y)
  function rpi2_epsilon_two(y,p)    
    implicit none
    real(kp) :: rpi2_epsilon_two
    real(kp), intent(in) :: y,p

    rpi2_epsilon_two = rpi_epsilon_two(y,p)

  end function rpi2_epsilon_two


  !epsilon_three(y)
  function rpi2_epsilon_three(y,p)    
    implicit none
    real(kp) :: rpi2_epsilon_three
    real(kp), intent(in) :: y,p

    rpi2_epsilon_three = rpi_epsilon_three(y,p)

  end function rpi2_epsilon_three



!maximal number of efolds computable at current numerical accuracy
  function rpi2_numacc_efoldmax(yend,p)
    implicit none
    real(kp) :: rpi2_numacc_efoldmax
    real(kp), intent(in) :: yend,p

    real(kp) :: yinimin
    
    
    yinimin = rpi_x_potmax(p)*(1._kp + epsilon(1._kp))
    
    rpi2_numacc_efoldmax = -rpi2_efold_primitive(yend,p) &
         + rpi2_efold_primitive(yinimin,p)

    
  end function rpi2_numacc_efoldmax
 

!minimal value of yendmin to get enough efold of inflation at this
!numerical accuracy
  function rpi2_numacc_xendmin(efold,p)
    implicit none
    real(kp) :: rpi2_numacc_xendmin
    real(kp), intent(in) :: efold,p

    real(kp) :: yinimin
        
    yinimin = rpi_x_potmax(p)*(1._kp+epsilon(1._kp))
        
    rpi2_numacc_xendmin = rpi2_x_trajectory(efold,yinimin,p)
    
    
  end function rpi2_numacc_xendmin

  

  !this is integral[V(phi)/V'(phi) dphi]
  function rpi2_efold_primitive(y,p)
    implicit none
    real(kp), intent(in) :: y,p
    real(kp) :: rpi2_efold_primitive

    real(kp) :: yVmax

    if (p.eq.1._kp) then
       rpi2_efold_primitive = rpih_efold_primitive(y,p)
       return
    endif


    yVmax = rpi_x_potmax(p)

    if (y.lt.yVmax) then
       write(*,*)'p= y= yVmax= ',p,y,yVmax
       stop 'rpi_efold_primitive: y < yVmax!'
    endif

    rpi2_efold_primitive = rpi_efold_primitive(y,p)
   
  end function rpi2_efold_primitive



  !returns y at bfold=-efolds before the end of inflation, ie N-Nend
  function rpi2_x_trajectory(bfold,yend,p)
    implicit none
    real(kp), intent(in) :: bfold, p, yend
    real(kp) :: rpi2_x_trajectory
    real(kp), parameter :: tolFind=tolkp
    real(kp) :: mini,maxi
    type(transfert) :: rpi2Data
    real(kp) :: yVmax


    if (p.eq.1._kp) then !Higgs Inflation Model (HI)
       rpi2_x_trajectory = rpih_x_trajectory(bfold,yend,p)
       return
    endif

    yVmax = rpi_x_potmax(p)

    if (yend.lt.yVmax) stop 'rpi2_x_trajectory: yend < yVmax'

    if (bfold.le.0._kp) then
       mini = yVmax*(1._kp+epsilon(1._kp))
       maxi = yEnd
    else
       mini = yEnd
       maxi = rpiBig       
    endif
    
    rpi2Data%real1 = p
    rpi2Data%real2 = -bfold + rpi2_efold_primitive(yend,p)
    rpi2Data%msg = 'rpi2_x_trajectory'
    
    rpi2_x_trajectory = zbrent(find_rpi_x_trajectory,mini,maxi,tolFind,rpi2Data)

  end function rpi2_x_trajectory


end module rpi2sr
