!Exact hubble flow functions and potential for Viatcheslav
!Fyodorovich Mukhanov Inflation (VFMI)
!
!For alpha not equal to 2 and not equal to 1:
!
!V(phi) = M^4 {1 - (1/2) beta/[1 + (x/2)(2-alpha)/sqrt(3 beta)]^[alpha/(2-alpha)]} 
! * exp( 3 beta/(1-alpha) * { [1 + (x/2)(2-alpha)/sqrt(3 beta)]^[2(1-alpha)/(2-alpha)]-1 })
!
!with the limiting values alpha->1
!
!V(phi) = M^4 (x)^(6 beta) [ 1-6 beta^2/x^2 ]
!
!and alpha->2
!
!V(phi) = M^4 {1 - (1/2) beta exp[-2 x/sqrt(3 beta)]} exp{-3 beta exp[-x/sqrt(3 beta)]}
!
!x = phi/Mpl

module vfmisr
  use infprec, only : kp

  implicit none

  private

  public vfmi_norm_potential, vfmi_norm_deriv_potential
  public vfmi_norm_deriv_second_potential, vfmi_deriv_ln_potential
  public vfmi_epsilon_one, vfmi_epsilon_two, vfmi_epsilon_three
  public vfmi_srepsilon_one, vfmi_srepsilon_two, vfmi_srepsilon_three
  public vfmi_efold_primitive, vfmi_x_trajectory, vfmi_x_endinf, vfmi_x_epsoneunity
  public vfmi_x_potmax, vfmi_numacc_betamax

contains


  function vfmi_numacc_betamax(efold,alpha)
    implicit none
    real(kp) :: vfmi_numacc_betamax
    real(kp), intent(in) :: efold, alpha

    real(kp), parameter :: lnPotNumAccMax = log(1._kp/epsilon(1._kp))

    if (alpha.gt.1._kp) then
       vfmi_numacc_betamax = huge(1._kp)
       return
    elseif (alpha.eq.1._kp) then
       vfmi_numacc_betamax = lnPotNumAccMax/3._kp/efold
       return
    endif

    vfmi_numacc_betamax = lnPotNumAccMax &
         * (1._kp - alpha)/3._kp/efold**(1._kp-alpha)

  end function vfmi_numacc_betamax




  function vfmi_norm_potential(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_norm_potential

    real(kp) :: y
    
    if (alpha.eq.1._kp) then

       y = 1._kp + 0.5_kp*x/sqrt(3._kp*beta)

       vfmi_norm_potential = y**(3._kp*beta) &
            *(1._kp-beta/y)
       return

    elseif (alpha.eq.2._kp) then

       y = exp(-x/sqrt(3._kp*beta))

       vfmi_norm_potential = (1._kp - 0.5_kp*beta*y*y) &
            * exp(-3._kp*beta*y)
       return

    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    vfmi_norm_potential = (1._kp - 0.5_kp*beta/y**(alpha/(2._kp-alpha))) &
         * exp(3._kp*beta/(1._kp-alpha)*(y**((1._kp-alpha)/(2._kp-alpha)) - 1._kp))
    
  end function vfmi_norm_potential



  function vfmi_norm_deriv_potential(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_norm_deriv_potential

    real(kp) :: y

    if (alpha.eq.1._kp) then

       y = 1._kp + 0.5_kp*x/sqrt(3._kp*beta)

       vfmi_norm_deriv_potential = (sqrt(beta)*y**(-1.5_kp + 3._kp*beta) &
            *(1._kp - 3._kp*beta + 3._kp*y))/sqrt(3._kp)

       return

    elseif (alpha.eq.2._kp) then

       y = exp(-x/sqrt(3._kp*beta))

       vfmi_norm_deriv_potential = -(sqrt(beta)*y*(-6._kp - 2._kp*y + 3._kp*beta*y**2)) &
            /(2._kp*sqrt(3._kp)*exp(3._kp*beta*(-1._kp + y)))


       return

    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    vfmi_norm_deriv_potential = -(exp((3._kp*beta*(-1._kp + y**((-1._kp + alpha) &
         /(-2._kp + alpha))))/(1._kp - alpha))*(-6._kp*sqrt(beta) &
         + sqrt(3._kp)*(-2._kp + alpha)*x)*y**(1._kp/(-2._kp + alpha)) &
         * (6._kp + alpha*y**(1._kp/(-2._kp + alpha)) &
         - 3._kp*beta*y**(alpha/(-2._kp + alpha))))/(12._kp*sqrt(3._kp))

  end function vfmi_norm_deriv_potential




  function vfmi_norm_deriv_second_potential(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_norm_deriv_second_potential

    real(kp) :: y

    if (alpha.eq.1._kp) then

       y = 1._kp + 0.5_kp*x/sqrt(3._kp*beta)

       vfmi_norm_deriv_second_potential = 0.5_kp*(y**(-2._kp + 3._kp*beta) &
            * (-1._kp + 5._kp*beta - 6._kp*beta**2 + (-1._kp + 6._kp*beta)*y))
       
       return

    elseif (alpha.eq.2._kp) then

       y = exp(-x/sqrt(3._kp*beta))

       vfmi_norm_deriv_second_potential = -(y*(6._kp + (4._kp - 18._kp*beta)*y &
            - 15._kp*beta*y**2 + 9._kp*beta**2*y**3))/(6._kp*exp(3*beta*(-1._kp + y)))

       return

    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    vfmi_norm_deriv_second_potential = -(exp((3._kp*beta*(-1._kp &
         + y**((-1._kp + alpha)/(-2._kp + alpha))))/(1._kp - alpha)) &
         * (6._kp*alpha*y**(1._kp/(-2._kp + alpha)) + alpha*(2._kp + alpha) &
         * y**(2._kp/(-2._kp + alpha)) - 36._kp*beta*y**(alpha/(-2._kp + alpha)) &
         + 18._kp*beta**2*y**((2._kp*alpha)/(-2._kp + alpha)) &
         - 15._kp*alpha*beta*y**((1._kp + alpha)/(-2._kp + alpha))))/12._kp

  end function vfmi_norm_deriv_second_potential



  function vfmi_deriv_ln_potential(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_deriv_ln_potential

    real(kp) :: z

    if (alpha.eq.2._kp) then
       z = exp(x/sqrt(3._kp*beta))
    else
       z = ( (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2 )**(1._kp/(2._kp-alpha))
    endif
       
    vfmi_deriv_ln_potential = (sqrt(beta)*z**(-1._kp - alpha/2._kp)*(3._kp*beta*z &
         - z**alpha*(alpha + 6._kp*z)))/(sqrt(3._kp)*(beta - 2._kp*z**alpha))

  end function vfmi_deriv_ln_potential


!first epsilon1V function (for test)
  function vfmi_srepsilon_one(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_srepsilon_one

    vfmi_srepsilon_one = 0.5_kp*vfmi_deriv_ln_potential(x,alpha,beta)**2

  end function vfmi_srepsilon_one

  
  function vfmi_srepsilon_two(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_srepsilon_two

    real(kp) :: z

    if (alpha.eq.2._kp) then
       z = exp(x/sqrt(3._kp*beta))
    else
       z = ( (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2 )**(1._kp/(2._kp-alpha))
    endif
    

    vfmi_srepsilon_two = (2._kp*alpha*sqrt(beta)*z**(-1._kp - alpha/2._kp)*(-3._kp*beta**2._kp*z &
         - 2._kp*z**(2._kp*alpha)*(2._kp+alpha+6._kp*z) + beta*z**alpha*(2._kp-alpha+12._kp*z))) &
         / ((-6._kp*sqrt(beta) + sqrt(3._kp)*(-2._kp+alpha)*x)*(beta-2._kp*z**alpha)**2._kp)

    if ((alpha.gt.2._kp).and.(x.gt.vfmi_x_potmax(alpha,beta))) then
       vfmi_srepsilon_two = -vfmi_srepsilon_two
    endif
    
  end function vfmi_srepsilon_two


  function vfmi_srepsilon_three(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x, alpha,beta
    real(kp) :: vfmi_srepsilon_three

    real(kp) :: z

    if (alpha.eq.2._kp) then
       z = exp(x/sqrt(3._kp*beta))
    else
       z = ( (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2)**(1._kp/(2._kp-alpha))
    endif
    
    vfmi_srepsilon_three = (2._kp*sqrt(beta)*z**(-1._kp-alpha/2._kp) &
         * (-3._kp*beta*z+z**alpha*(alpha+6._kp*z))*(-3._kp*beta**3._kp*z+8._kp*z**(3._kp*alpha) &
         * (2._kp+alpha+3._kp*z) + beta**2._kp*z**alpha*((-2._kp+alpha)**2._kp+18._kp*z) &
         - 2._kp*beta*z**(2._kp*alpha)*(8._kp-alpha*(2._kp+3._kp*alpha)+18._kp*z))) &
         / ((6._kp*sqrt(beta)-sqrt(3._kp)*(-2._kp+alpha)*x)*(beta-2._kp*z**alpha)**2._kp &
         * (3._kp*beta**2._kp*z+2._kp*z**(2._kp*alpha)*(2._kp+alpha+6._kp*z) &
         - beta*z**alpha*(2._kp-alpha+12._kp*z)))

    if ((alpha.gt.2._kp).and.(x.gt.vfmi_x_potmax(alpha,beta))) then
       vfmi_srepsilon_three = -vfmi_srepsilon_three
    endif
    
  end function vfmi_srepsilon_three
  

  
!first hubble flow function (exact)
  function vfmi_epsilon_one(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_epsilon_one

    real(kp) :: y

    if (alpha.eq.2._kp) then
       vfmi_epsilon_one = 1.5_kp*beta*exp(-2._kp*x/sqrt(3._kp*beta))
       return
    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    vfmi_epsilon_one = 1.5_kp*beta/y**(alpha/(2._kp-alpha))
        
  end function vfmi_epsilon_one




!second hubble flow function (exact)
  function vfmi_epsilon_two(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_epsilon_two

    real(kp) :: y

    if (alpha.eq.2._kp) then
       vfmi_epsilon_two = 2._kp*exp(-x/sqrt(3._kp*beta))
       return
    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    vfmi_epsilon_two = alpha/y**(1._kp/(2._kp-alpha))

  end function vfmi_epsilon_two
 



!third (and all higher order) hubble flow function (exact)
  function vfmi_epsilon_three(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_epsilon_three

    vfmi_epsilon_three = vfmi_epsilon_two(x,alpha,beta)/alpha

  end function vfmi_epsilon_three


  function vfmi_x_epsoneunity(alpha,beta)
    implicit none
    real(kp), intent(in) :: alpha,beta
    real(kp), dimension(2) :: vfmi_x_epsoneunity

    if (alpha.eq.2._kp) then
       vfmi_x_epsoneunity = 0.5_kp*sqrt(3._kp*beta)*log(1.5_kp*beta)
       return
    endif

    vfmi_x_epsoneunity(1) = 2._kp*sqrt(3._kp*beta)/(2._kp-alpha) &
         * ((1.5_kp*beta)**((2._kp-alpha)/alpha/2._kp) - 1._kp)

    if (alpha.gt.2._kp) then
       vfmi_x_epsoneunity(2) = 2._kp*sqrt(3._kp*beta)/(2._kp-alpha) &
            * (-(1.5_kp*beta)**((2._kp-alpha)/alpha/2._kp) - 1._kp)
    else
       vfmi_x_epsoneunity(2) = vfmi_x_epsoneunity(1)
    endif
        
  end function vfmi_x_epsoneunity
  


!returns the field value at which inflation ends (left of the potential)
  function vfmi_x_endinf(alpha,beta)
    implicit none
    real(kp), intent(in) :: alpha,beta
    real(kp), dimension(2) :: xepsone
    real(kp) :: vfmi_x_endinf
    
    xepsone = vfmi_x_epsoneunity(alpha,beta)

    vfmi_x_endinf = xepsone(1)

  end function vfmi_x_endinf


  

!return the field value at which the potential is maximal (exist only
!for alpha > 2)
  function vfmi_x_potmax(alpha,beta)
    implicit none
    real(kp), intent(in) :: alpha,beta
    real(kp) :: vfmi_x_potmax

    if (alpha.le.2._kp) then
       stop 'vfmi_x_potmax: exists only for alpha > 2'
    endif

    vfmi_x_potmax = 2._kp*sqrt(3._kp*beta)/(alpha-2._kp)

  end function vfmi_x_potmax



  function vfmi_efold_primitive(x,alpha,beta)
    implicit none
    real(kp), intent(in) :: x,alpha,beta
    real(kp) :: vfmi_efold_primitive

    real(kp) :: y, bfold

    if (alpha.eq.2._kp) then
       bfold = sqrt(1.5_kp*beta) - exp(x/sqrt(3._kp*beta))
       vfmi_efold_primitive = - bfold
       return
    endif

    y = (1._kp + x * (2._kp-alpha)/sqrt(3._kp*beta)/2._kp)**2

    bfold = (1.5_kp*beta)**(1._kp/alpha) - y**(1._kp/(2._kp-alpha))

    vfmi_efold_primitive = -bfold      

  end function vfmi_efold_primitive



!if you enter xend>xvmax with alpha>2, the trajectory describes the
!upper branch
  function vfmi_x_trajectory(bfold,xend,alpha,beta)
    implicit none
    real(kp), intent(in) :: bfold,xend,alpha,beta
    real(kp) :: vfmi_x_trajectory
    real(kp) :: csign

    if (alpha.eq.2._kp) then
       vfmi_x_trajectory = sqrt(3._kp*beta)*log(sqrt(1.5_kp*beta) - bfold)
       return
    endif

    csign = 1._kp

!increasing field trajectory at x>xvmax    
    if ((alpha.gt.2._kp).and.(xend.gt.vfmi_x_potmax(alpha,beta))) then
       csign = -1._kp
    end if
           
    vfmi_x_trajectory = 2._kp*sqrt(3._kp*beta)/(2._kp-alpha) &
         * ( csign*((1.5_kp*beta)**(1._kp/alpha) - bfold)**(1._kp-0.5_kp*alpha) - 1._kp )

 end function vfmi_x_trajectory




end module vfmisr
